# coding: utf-8
from quokka.template import render_template


def configure(app):
    if app.config.get('DEBUG') is True:
        app.logger.debug('Skipping error handlers in Debug mode')
        return

    @app.errorhandler(403)
    def forbidden_page(*args, **kwargs):
        """
        The server understood the request, but is refusing to fulfill it.
        Authorization will not help and the request SHOULD NOT be repeated.
        If the request method was not HEAD and the server wishes to make public
        why the request has not been fulfilled, it SHOULD describe the
        reason for
        the refusal in the entity. If the server does not wish to make this
        information available to the client, the status code 404 (Not Found)
        can be used instead.
        """
        return render_template("errors/access_forbidden.html"), 403

    @app.errorhandler(404)
    def page_not_found(*args, **kwargs):
        """
        The server has not found anything matching the Request-URI.
        No indication
        is given of whether the condition is temporary or permanent.
        The 410 (Gone)
        status code SHOULD be used if the server knows, through some internally
        configurable mechanism, that an old resource is permanently unavailable
        and has no forwarding address. This status code is commonly used when
        the
        server does not wish to reveal exactly why the request has been
        refused,
        or when no other response is applicable.
        """
        return render_template("errors/page_not_found.html"), 404

    @app.errorhandler(405)
    def method_not_allowed_page(*args, **kwargs):
        """
        The method specified in the Request-Line is not allowed for the
        resource
        identified by the Request-URI. The response MUST include an
        Allow header
        containing a list of valid methods for the requested resource.
        """
        return render_template("errors/method_not_allowed.html"), 405

    @app.errorhandler(500)
    def server_error_page(*args, **kwargs):
        return render_template("errors/server_error.html"), 500

    # URLBUILD Error Handlers
    def admin_icons_error_handler(error, endpoint, *args, **kwargs):
        "when some of default dashboard button is deactivated, avoids error"
        if endpoint in [item[0] for item in app.config.get('ADMIN_ICONS', [])]:
            return '/admin'
        raise error

    app.url_build_error_handlers.append(admin_icons_error_handler)




from flask import current_app
from quokka.template import render_template
from wtforms.widgets import TextArea, TextInput


class TextEditor(TextArea):
    def __init__(self, *args, **kwargs):
        super(TextEditor, self).__init__()
        self.rows = kwargs.get('rows', 20)
        self.cols = kwargs.get('cols', 20)
        self.css_cls = kwargs.get('css_cls', 'text_editor')
        self.style_ = kwargs.get(
            'style_',
            "margin: 0px; width: 725px; height: 360px;"
        )
        self.editor = kwargs.pop('editor', 'texteditor')

    def __call__(self, field, **kwargs):
        c = kwargs.pop('class', '') or kwargs.pop('class_', '')
        kwargs['class'] = u'%s %s' % (self.css_cls, c)
        kwargs['rows'] = self.rows
        kwargs['cols'] = self.cols
        s = kwargs.pop('style', '') or kwargs.pop('style_', '')
        kwargs['style'] = u"%s %s" % (self.style_, s)
        html = super(TextEditor, self).__call__(field, **kwargs)
        html += render_template(
            'admin/texteditor/%s.html' % self.editor,
            theme=current_app.config.get('ADMIN_THEME', 'admin'),
            selector='.' + self.css_cls
        )
        return html


class PrepopulatedText(TextInput):
    def __init__(self, *args, **kwargs):
        self.master = kwargs.pop('master', '')
        super(PrepopulatedText, self).__init__(*args, **kwargs)

    def __call__(self, *args, **kwargs):
        html = super(PrepopulatedText, self).__call__(*args, **kwargs)
        slave = args[0].id
        if self.master:
            html += render_template(
                'admin/custom/prepopulated.html',
                theme=current_app.config.get('ADMIN_THEME', 'admin'),
                master=self.master,
                slave=slave
            )
        return html



# coding: utf-8

try:
    from flask_debugtoolbar import DebugToolbarExtension
except ImportError:
    DebugToolbarExtension = None

try:
    from opbeat.contrib.flask import Opbeat
except ImportError:
    Opbeat = None

try:
    from raven.contrib.flask import Sentry
except ImportError:
    Sentry = None


def configure(app, admin=None):
    if app.config.get('DEBUG_TOOLBAR_ENABLED'):
        try:
            DebugToolbarExtension(app)
        except TypeError:
            raise ImportError('You must install flask_debugtoolbar')

    if app.config.get('OPBEAT'):
        try:
            Opbeat(
                app,
                logging=app.config.get('OPBEAT', {}).get('LOGGING', False)
            )
            app.logger.info('opbeat configured!!!')
        except TypeError:
            raise ImportError('You must install opbeat')

    if app.config.get('SENTRY_ENABLED', False):
        try:
            app.sentry = Sentry(app)
        except TypeError:
            raise ImportError('You must install raven (Sentry)')



# coding: utf-8
import json
import logging
import uuid

from quokka.core.models.channel import Channel, ChannelType
from quokka.core.models.config import Config, Quokka
from quokka.core.models.content import License
from quokka.core.models.custom_values import CustomValue
from quokka.core.models.subcontent import SubContentPurpose
from quokka.modules.accounts.models import Role, User
from quokka.modules.posts.models import Post

logger = logging.getLogger()


class Populate(object):
    def __init__(self, db, *args, **kwargs):
        self.db = db
        self.args = args
        self.kwargs = kwargs
        self.roles = {}
        self.users = {}
        self.channels = {}
        self.channel_types = {}
        self.purposes = {}
        self.custom_values = {}
        self.load_fixtures()
        self.baseurl = self.kwargs.get('baseurl', 'http://localhost:5000')
        self.app = self.kwargs.get('app')

    def __call__(self, *args, **kwargs):
        if self.baseurl and self.app:
            with self.app.test_request_context(base_url=self.baseurl):
                self.pipeline()
        else:
            self.pipeline()

    def pipeline(self):
        self.load_existing_users()
        self.create_users()
        self.create_configs()
        self.create_channel_types()
        self.create_base_channels()
        self.create_channels()
        self.create_purposes()
        self.create_posts()

    def generate_random_password(self):
        return uuid.uuid4().hex

    def create_initial_superuser(self):
        password = self.generate_random_password()
        user_data = {
            "name": "Quokka Admin",
            "email": "admin@quokkaproject.org",
            "gravatar_email": "rochacbruno+quokka@gmail.com",
            "password": password[:6],
            "roles": ["admin"],
            "bio": "Quokka Example Admin",
            "tagline": "Quokka is the best CMS!",
            "links": [
                {
                    "title": "facebook",
                    "link": "http://facebook.com/quokkaproject",
                    "icon": "facebook",
                    "css_class": "facebook",
                    "order": 0
                },
                {
                    "title": "github",
                    "link": "http://github.com/quokkaproject",
                    "icon": "github",
                    "css_class": "github",
                    "order": 0
                },
                {
                    "title": "twitter",
                    "link": "http://twitter.com/quokkaproject",
                    "icon": "twitter",
                    "css_class": "twitter",
                    "order": 0
                }
            ]
        }
        user_obj = self.create_user(user_data)
        return user_data, user_obj

    def load_fixtures(self):
        filepath = self.kwargs.get('filepath',
                                   './etc/fixtures/initial_data.json')
        _file = open(filepath)
        self.json_data = json.load(_file)

    def role(self, name):
        if name not in self.roles:
            if isinstance(name, Role):
                logger.info('get existing role: %s', name)
                self.roles[name.name] = name
                role = name
            else:
                role, created = Role.objects.get_or_create(name=name)
                self.roles[name] = role
                if created:
                    logger.info("Created role: %s", name)
        else:
            role = self.roles.get(name)

        if not isinstance(role, Role):
            raise TypeError(
                u"%s should be a Role and is %s" % (role, type(role))
            )

        return role

    def load_existing_users(self):
        users = User.objects.all()
        for user in users:
            self.users[user.name] = user

    def create_user(self, data):
        name = data.get('name')
        if name not in self.users:
            pwd = data.get("password")
            data['roles'] = [self.role(role) for role in data.get('roles')]
            user = User.createuser(**data)
            self.users[name] = user
            logger.info("Created: User: mail:%s pwd:%s", user.email, pwd)
            return user
        else:
            logger.info("Exist: User: mail: %s", data.get('email'))

    def create_users(self, data=None):
        self.users_data = data or self.json_data.get('users')
        for data in self.users_data:
            self.create_user(data)

    @staticmethod
    def create_config(data):
        try:
            return Config.objects.get(group=data.get('group'))
        except:
            return Config.objects.create(**data)

    def custom_value(self, **data):
        if data.get('name') in self.custom_values:
            return self.custom_values[data.get('name')]

        value = CustomValue(**data)
        self.custom_values[value.name] = value
        return value

    def create_configs(self):

        self.configs_data = self.json_data.get('configs')

        for config in self.configs_data:
            config['values'] = [self.custom_value(**args)
                                for args in config.get('values')]
            self.create_config(config)

    def create_channel(self, data):

        if 'childs' in data:
            childs = data.pop('childs')
        else:
            childs = []

        data['created_by'] = data['last_updated_by'] = self.users.get('admin')
        _type = data.get('channel_type')
        data['channel_type'] = self.channel_types.get(_type)

        try:
            channel = Channel.objects.get(slug=data.get('slug'))
            created = False
        except:
            channel, created = Channel.objects.get_or_create(**data)

        if created:
            logger.info("Created channel: %s", channel.title)
        else:
            logger.info("Channel get: %s", channel.title)

        for child in childs:
            child['parent'] = channel
            self.create_channel(child)

        if channel.slug not in self.channels:
            self.channels[channel.slug] = channel

        return channel

    def create_channel_type(self, data):
        try:
            channel_type = ChannelType.objects.get(
                identifier=data.get('identifier'))
            created = False
        except:
            channel_type, created = ChannelType.objects.get_or_create(
                **data
            )

        if created:
            logger.info("Created channel_type: %s", channel_type.title)
        else:
            logger.info("ChannelType get: %s", channel_type.title)

        if channel_type.identifier not in self.channel_types:
            self.channel_types[channel_type.identifier] = channel_type

        return channel_type

    def create_base_channels(self):
        self.channel_data = self.json_data.get('base_channels')
        for data in self.channel_data:
            self.create_channel(data)

    def create_channels(self):
        self.channel_data = self.json_data.get('channels')
        for data in self.channel_data:
            self.create_channel(data)

    def create_channel_types(self):
        self.channel_type_data = self.json_data.get('channel_types')

        for data in self.channel_type_data:
            self.create_channel_type(data)

    def create_purpose(self, data):
        if data.get('identifier') in self.purposes:
            return self.purposes[data.get('identifier')]

        purpose, created = SubContentPurpose.objects.get_or_create(
            title=data.get('title'),
            identifier=data.get('identifier'),
            module=data.get('module')
        )

        self.purposes[purpose.identifier] = purpose

        return purpose

    def create_purposes(self):
        self.purpose_data = self.json_data.get('purposes')
        for purpose in self.purpose_data:
            self.create_purpose(purpose)

    def create_initial_post(self, user_data=None, user_obj=None):
        post_data = dict(
            title="Try Quokka CMS! write a post.",
            summary=(
                "Use default credentials to access "
                "/admin \r\n"
                "user: {user[email]} \r\n"
                "pass: {user[password]} \r\n"
            ).format(user=user_data),
            slug="try-quokka-cms",
            tags=["quokka"],
            body=(
                 "## You can try Quokka ADMIN\r\n\r\n"
                 "Create some posts\r\n\r\n"
                 "> Use default credentials to access "
                 "[/admin](/admin) \r\n\r\n"
                 "- user: {user[email]}\r\n"
                 "- password: {user[password]}\r\n"
                 "> ATTENTION! Copy the credentials and delete this post"
            ).format(user=user_data),
            license=dict(
                title="Creative Commons",
                link="http://creativecommons.com",
                identifier="creative_commons_by_nc_nd"
            ),
            content_format="markdown"
        )
        post_data['channel'] = self.channels.get("home")
        post_data["created_by"] = user_obj or self.users.get('author')
        if self.users.get('author2'):
            post_data["authors"] = [self.users.get('author2')]
        post = self.create_post(post_data)
        return post

    def create_post(self, data):
        created_by = data.get('created_by')
        if not created_by:
            data['created_by'] = self.users.get('author')
        else:
            if isinstance(created_by, User):
                data['created_by'] = created_by
            else:
                data['created_by'] = self.users.get(created_by)

        data['last_updated_by'] = data['created_by']

        if data.get('authors'):
            authors = [self.users.get(user) for user in data.get('authors')]
            data["authors"] = [user for user in authors if user is not None]

        data['published'] = True

        if 'license' in data and not isinstance(data['license'], License):
            data['license'] = License(**data['license'])

        try:
            post = Post.objects.get(slug=data.get('slug'))
            logger.info("Post get: %s", post.title)
        except:
            post = Post.objects.create(**data)
            logger.info("Post created: %s", post.title)

        return post

    def create_posts(self):
        self.post_data = self.json_data.get('posts')
        for data in self.post_data:
            _channel = data.get('channel')
            data['channel'] = self.channels.get(_channel)

            related_channels = data.get('related_channels', [])
            data['related_channels'] = [
                self.channels.get(_related)
                for _related in related_channels
            ]

            try:
                self.create_post(data)
            except:
                self.create_channels()
                self.create_post(data)

    def reset(self):
        Post.objects(
            slug__in=[item['slug'] for item in self.json_data.get('posts')]
        ).delete()

        SubContentPurpose.objects(
            identifier__in=[
                item['identifier'] for item in self.json_data.get('purposes')
            ]
        ).delete()

        for channel in Channel.objects(
                slug__in=[
                    item['slug'] for item in self.json_data.get('channels')]):
            for subchannel in channel.get_children():
                for subsubchannel in subchannel.get_children():
                    subsubchannel.delete()
                subchannel.delete()
            channel.delete()

        User.objects(
            email__in=[item['email'] for item in self.json_data.get('users')]
        ).delete()

        if self.kwargs.get('first_install'):
            Quokka.objects.delete()



# -*- coding: UTF-8 -*-

"""
Generates render keywords for `WTForms`_ HTML5 field's widgets.

The :func:`get_html5_kwargs` adds the automatically generated keys to the
*render keywords* of a `WTForms`_ field.

The :cls:`AutoAttrMeta` can be included as a base class for the `Meta` class
in your forms, to handle this automatically for each field of the form.


Supported Auto–Attributes
-------------------------

- *required*

  Is set if the field has the _required_ flag set. This happens i.e. if you
  use the _DataRequired_ or _InputRequired_ validator. The `required`
  attribute is used by browsers to indicate a required field (and most
  browsers won't activate the forms action unless all required fields have
  content).

- *invalid*

  If the field got any validation errors, the CSS class _invalid_ is added.
  The `invalid` class is also set by browsers if they detect errors on a
  field. This validation errors detected by your code, are by default styled
  in the same way as browser generated errors.

- *min* and *max*

  If either _Length_ or _NumberRange_ is used as a validator to set minimal
  and / or maximal values, the corresponding `min` / `max` INPUT attribute is
  set. This allows for browser based validation of the values.

- *title*

  If no _title_ is provided for a field, the _description_ (if one is set) is
  used for the `title` attribute.


An Example
----------

Declare your form just like in vanilla *WTForms*, but include `AutoAttrMeta`
as your meta class:

>>> from wtforms import Form, StringField
>>> from wtforms.validators import InputRequired, Length
>>> from wtforms_html5 import AutoAttrMeta

>>> class MyForm(Form):
...   class Meta(AutoAttrMeta):
...     pass
...   test_field = StringField(
...     'Testfield',
...      validators=[InputRequired(), Length(min=3, max=12)],
...      description='Just a test field.',
...   )

>>> form = MyForm()

**The only difference is, that you include a `Meta` class, that inherits from
`AutoAttrMeta`.**

This meta class sets the above mentioned attributes automatically for all the
fields of the form:

>>> f = form.test_field()
>>> exp = (
...  '<input id="test_field" max="12" min="3" name="test_field" required '
...  'title="Just a test field." type="text" value="">'
... )
>>> assert f == exp



The _min_ and _max_ attributes are created because the `Length` validator was
used. And the field is marked _required_ because of the `InputRequired`
validator. The field also gets a _title_ taken from the fields `description`.

If you validate the form and any errors pop up, the field also get `invalid`
added to its class:

>>> form.validate()
False
>>> exp = (
... '<input class="invalid" id="test_field" max="12" min="3" '
... 'name="test_field" required title="Just a test field." type="text" '
... 'value="">'
... )
>>> f = form.test_field()
>>> assert f == exp


.. _WTForms: https://wtforms.readthedocs.io/

"""

from __future__ import absolute_import, unicode_literals

from wtforms.fields.core import UnboundField
from wtforms.meta import DefaultMeta
from wtforms.validators import Length, NumberRange

__version__ = '0.3.0'
__author__ = 'Brutus [DMC] <brutus.dmc@googlemail.com>'
__license__ = 'GNU General Public License v3 or above - '\
              'http://www.opensource.org/licenses/gpl-3.0.html'


MINMAX_VALIDATORS = (
    Length,
    NumberRange,
)


def set_required(field, render_kw=None, force=False):
    """
    Returns *render_kw* with *required* set if the field is required.

    Sets the *required* key if the `required` flag is set for the field (this
    is mostly the case if it is set by validators). The `required` attribute
    is used by browsers to indicate a required field.

    ..note::

        This won't change keys already present unless *force* is used.

    """
    if render_kw is None:
        render_kw = {}
    if 'required' in render_kw and not force:
        return render_kw
    if field.flags.required:
        render_kw['required'] = True
    return render_kw


def set_invalid(field, render_kw=None):
    """
    Returns *render_kw* with `invalid` added to *class* on validation errors.

    Set (or appends) 'invalid' to the fields CSS class(es), if the *field* got
    any errors. 'invalid' is also set by browsers if they detect errors on a
    field.

    """
    if render_kw is None:
        render_kw = {}
    if field.errors:
        classes = render_kw.get('class') or render_kw.pop('class_', '')
        if classes:
            render_kw['class'] = 'invalid {}'.format(classes)
        else:
            render_kw['class'] = 'invalid'
    return render_kw


def set_minmax(field, render_kw=None, force=False):
    """
    Returns *render_kw* with *min* and *max* set if validators use them.

    Sets *min* and / or *max* keys if a `Length` or `NumberRange` validator is
    using them.

    ..note::

        This won't change keys already present unless *force* is used.

    """
    if render_kw is None:
        render_kw = {}
    for validator in field.validators:
        if isinstance(validator, MINMAX_VALIDATORS):
            if 'min' not in render_kw or force:
                v_min = getattr(validator, 'min', -1)
                if v_min not in (-1, None):
                    render_kw['min'] = v_min
            if 'max' not in render_kw or force:
                v_max = getattr(validator, 'max', -1)
                if v_max not in (-1, None):
                    render_kw['max'] = v_max
    return render_kw


def set_title(field, render_kw=None):
    """
    Returns *render_kw* with *min* and *max* set if required.

    If the field got a *description* but no *title* key is set, the *title* is
    set to *description*.

    """
    if render_kw is None:
        render_kw = {}
    if 'title' not in render_kw and getattr(field, 'description'):
        render_kw['title'] = '{}'.format(field.description)
    return render_kw


def get_html5_kwargs(field, render_kw=None, force=False):
    """
    Returns a copy of *render_kw*  with keys added for a bound *field*.

    If some *render_kw* are given, the new keys are added to a copy of them,
    which is then returned. If none are given, a dictionary containing only
    the automatically generated keys is returned.

    .. important::

        This might add new keys but won't changes any values if a key is
        already in *render_kw*,  unless *force* is used.

    Raises:

        ValueError: if *field* is an :cls:`UnboundField`.

    The following keys are set automatically:

    :required:
        Sets the *required* key if the `required` flag is set for the
        field (this is mostly the case if it is set by validators). The
        `required` attribute is used by browsers to indicate a required field.

    :invalid:
        Set (or appends) 'invalid' to the fields CSS class(es), if the *field*
        got any errors. 'invalid' is also set by browsers if they detect
        errors on a field.

    :min / max:
        Sets *min* and / or *max* keys if a `Length` or `NumberRange`
        validator is using them.

    :title:
        If the field got a *description* but no *title* key is set, the
        *title* is set to *description*.

    """
    if isinstance(field, UnboundField):
        msg = 'This function needs a bound field not: {}'
        raise ValueError(msg.format(field))
    kwargs = render_kw.copy() if render_kw else {}
    kwargs = set_required(field, kwargs, force)  # is field required?
    kwargs = set_invalid(field, kwargs)  # is field invalid?
    kwargs = set_minmax(field, kwargs, force)  # check validators for min/max
    kwargs = set_title(field, kwargs)  # missing tile?
    return kwargs


class AutoAttrMeta(DefaultMeta):
    """
    Meta class for WTForms :cls:`Form` classes.

    It uses :func:`get_html5_kwargs` to automatically add some render
    keywords for each field's widget when it gets rendered.

    """

    def render_field(self, field, render_kw):
        """
        Returns the rendered field after adding auto–attributes.

        Calls the field`s widget with the following kwargs:

        1. the *render_kw* set on the field are used as based
        2. and are updated with the *render_kw* arguments from the render call
        3. this is used as an argument for a call to `get_html5_kwargs`
        4. the return value of the call is used as final *render_kw*

        """
        field_kw = getattr(field, 'render_kw', None)
        if field_kw is not None:
            render_kw = dict(field_kw, **render_kw)
        render_kw = get_html5_kwargs(field, render_kw)
        return field.widget(field, **render_kw)



import importlib
import os
import sys

import click


class CommandsCollector(click.MultiCommand):
    """A MultiCommand to collect all click commands from a given
    modules path and base name for the module.
    The commands functions needs to be in a module inside commands
    folder and the name of the file will be used as the command name.
    """

    def __init__(self, modules_path, base_module_name, **attrs):
        click.MultiCommand.__init__(self, **attrs)
        self.base_module_name = base_module_name
        self.modules_path = modules_path

    def list_commands(self, *args, **kwargs):
        commands = []
        for _path, _dir, _ in os.walk(self.modules_path):
            if 'commands' not in _dir:
                continue
            for filename in os.listdir(os.path.join(_path, 'commands')):
                if filename.endswith('.py') and filename != '__init__.py':
                    cmd = filename[:-3]
                    _, module_name = os.path.split(_path)
                    commands.append('{0}_{1}'.format(module_name, cmd))
        commands.sort()
        return commands

    def get_command(self, ctx, name):
        try:
            if sys.version_info[0] == 2:
                name = name.encode('ascii', 'replace')
            splitted = name.split('_')
            if len(splitted) <= 1:
                return
            module_name, command_name = splitted
            if not all([module_name, command_name]):
                return
            module = '{0}.{1}.commands.{2}'.format(
                self.base_module_name,
                module_name,
                command_name)
            mod = importlib.import_module(module)
        except ImportError:
            return
        return getattr(mod, 'cli', None)



# coding: utf-8

import datetime

from quokka.core.models.channel import Channel
from quokka.core.models.config import Config
from quokka.core.models.content import Content, Link


def configure(app):

    @app.context_processor
    def inject():
        now = datetime.datetime.now()
        return dict(
            channels=Channel.objects(published=True,
                                     available_at__lte=now,
                                     parent=None),
            Config=Config,
            Content=Content,
            Channel=Channel,
            homepage=Channel.get_homepage(),
            Link=Link
        )



# -*- coding: utf-8 -*-

from flask import current_app, session
from quokka_themes import render_theme_template


def render_template(template, theme=None, **context):
    theme = theme or []
    if not isinstance(theme, (list, tuple)):
        theme = [theme]

    sys_theme = session.get('theme', current_app.config.get('DEFAULT_THEME'))
    if sys_theme:
        theme.append(sys_theme)

    return render_theme_template(theme, template, **context)



"""

"""
from flask_admin.contrib.pymongo import filters
from flask_admin.form import Select2Widget
from flask_admin.model.fields import InlineFieldList, InlineFormField
# from wtforms import fields, form
from quokka.admin.forms import Form, fields
from quokka.admin.views import ModelView
from quokka.db import collection_users, connection


# User admin
class InnerForm(Form):
    username = fields.StringField('Username')
    test = fields.StringField('Test')


class UserForm(Form):
    username = fields.StringField('Username')
    email = fields.StringField('Email')
    password = fields.StringField('Password')

    # Inner form
    inner = InlineFormField(InnerForm)

    # Form list
    form_list = InlineFieldList(InlineFormField(InnerForm))


class UserView(ModelView):
    column_list = ('username', 'email', 'password')
    column_sortable_list = ('username', 'email', 'password')

    form = UserForm

    page_size = 20
    can_set_page_size = True

    # Correct user_id reference before saving
    def on_model_change(self, form, model):
        model['_id'] = model.get('username')
        return model


# Tweet view
class TweetForm(Form):
    name = fields.StringField('Name')
    user_id = fields.SelectField('User', widget=Select2Widget())
    text = fields.StringField('Text')
    testie = fields.BooleanField('Test')


class TweetView(ModelView):
    column_list = ('name', 'user_name', 'text')
    column_sortable_list = ('name', 'text')

    column_filters = (filters.FilterEqual('name', 'Name'),
                      filters.FilterNotEqual('name', 'Name'),
                      filters.FilterLike('name', 'Name'),
                      filters.FilterNotLike('name', 'Name'),
                      filters.BooleanEqualFilter('testie', 'Testie'))

    # column_searchable_list = ('name', 'text')

    form = TweetForm

    def get_list(self, *args, **kwargs):
        # not necessary but kept as example
        count, data = super(TweetView, self).get_list(*args, **kwargs)

        # Contribute user_name to the models
        for item in data:
            user = collection_users.find_one(
                {'_id': item['user_id']}
            )
            if user:
                item['user_name'] = user['_id']

        return count, data

    # Contribute list of user choices to the forms
    def _feed_user_choices(self, form):
        users = collection_users.find(fields=('_id',))
        form.user_id.choices = [(str(x['_id']), x['_id']) for x in users]
        return form

    def create_form(self):
        form = super(TweetView, self).create_form()
        return self._feed_user_choices(form)

    def edit_form(self, obj):
        form = super(TweetView, self).edit_form(obj)
        return self._feed_user_choices(form)


def configure(app, db, admin):
    admin.register(
        collection_users,
        UserView,
        # category='User',
        name='User'
    )

    admin.register(
        connection['tweets'].tweets,
        TweetView,
        # category='User',
        name='Tweets'
    )



from pymongo import MongoClient
from tinydb_serialization import SerializationMiddleware
from tinymongo import TinyMongoClient
from tinymongo.serializers import DateTimeSerializer


class QuokkaTinyMongoClient(TinyMongoClient):
    @property
    def _storage(self):
        serialization = SerializationMiddleware()
        serialization.register_serializer(DateTimeSerializer(), 'TinyDate')
        # TODO: Read custom serializers from settings and extensions
        return serialization


class QuokkaDB:
    def __init__(self, app=None):
        self.app = None
        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        self.config = app.config.get('DATABASE', {})
        self.system = self.config.get('system', 'tinydb')
        self.folder = self.config.get('folder', 'databases')
        self.username = self.config.get('username')
        self.password = self.config.get('password')
        self.host = self.config.get('host', 'localhost')
        self.port = self.config.get('port', 'port')
        self.name = self.config.get('name', 'quokka_db')
        self.collections = {
            'index': 'index',
            'contents': 'contents',
            'uploads': 'uploads',
            'users': 'users',
        }
        self.collections.update(self.config.get('collections', {}))
        self._register(app)

    def _register(self, app):
        if not hasattr(app, 'extensions'):
            app.extensions = {}

        if 'db' in app.extensions:
            raise RuntimeError("Flask extension already initialized")

        app.extensions['db'] = self
        self.app = app

    def get_db_name(self, collection):
        """return db_name for collection"""
        if self.system == "mongo":
            return self.name
        return collection

    def get_collection(self, collection):
        col_name = self.collections[collection]
        db_name = self.get_db_name(col_name)
        return self.connection[db_name][col_name]

    @property
    def connection(self):
        if getattr(self, '_connection', None) is None:
            if self.system == 'tinydb':
                self._connection = QuokkaTinyMongoClient(self.folder)
            elif self.system == 'mongo':
                self._connection = MongoClient(
                    host=self.host,
                    port=self.port,
                    username=self.username,
                    password=self.password
                )
        return self._connection

    @property
    def index(self):
        return self.get_collection('index')

    @property
    def contents(self):
        return self.get_collection('contents')

    @property
    def uploads(self):
        return self.get_collection('uploads')

    @property
    def users(self):
        return self.get_collection('users')




from flask_htmlbuilder.htmlbuilder import html
from quokka.admin.utils import _l


def format_datetime(self, request, obj, fieldname, *args, **kwargs):
    return html.div(style='min-width:130px;')(
        getattr(obj, fieldname).strftime(self.get_datetime_format())
    )


def view_on_site(self, request, obj, fieldname, *args, **kwargs):
    available = obj.is_available
    endpoint = kwargs.pop(
        'endpoint',
        'quokka.core.detail' if available else 'quokka.core.preview'
    )
    return html.a(
        href=obj.get_absolute_url(endpoint),
        target='_blank',
    )(html.i(class_="icon icon-eye-open", style="margin-right: 5px;")(),
      _l('View on site') if available else _l('Preview on site'))


def format_ul(self, request, obj, fieldname, *args, **kwars):
    field = getattr(obj, fieldname)
    column_formatters_args = getattr(self, 'column_formatters_args', {})
    _args = column_formatters_args.get('ul', {}).get(fieldname, {})
    ul = html.ul(style=_args.get('style', "min-width:200px;max-width:300px;"))
    placeholder = _args.get('placeholder', u"{i}")
    lis = [html.li(placeholder.format(item=item)) for item in field]
    return ul(*lis)


def format_link(self, request, obj, fieldname, *args, **kwars):
    value = getattr(obj, fieldname)
    return html.a(href=value, title=value, target='_blank')(
        html.i(class_="icon  icon-resize-small",
               style="margin-right: 5px;")()
    )


def format_status(self, request, obj, fieldname, *args, **kwargs):
    status = getattr(obj, fieldname)
    column_formatters_args = getattr(self, 'column_formatters_args', {})
    _args = column_formatters_args.get('status', {}).get(fieldname, {})
    labels = _args.get('labels', {})
    return html.span(
        class_="label label-{0}".format(labels.get(status, 'default')),
        style=_args.get('style', 'min-height:18px;')
    )(status)


def get_url(self, request, obj, fieldname, *args, **kwargs):
    column_formatters_args = getattr(self, 'column_formatters_args', {})
    _args = column_formatters_args.get('get_url', {}).get(fieldname, {})
    attribute = _args.get('attribute')
    method = _args.get('method', 'get_absolute_url')
    text = getattr(obj, fieldname, '')
    if attribute:
        target = getattr(obj, attribute, None)
    else:
        target = obj

    url = getattr(target, method, lambda: '#')()

    return html.a(href=url)(text if text not in [None, 'None'] else '')



# coding: utf-8

from flask import current_app, flash, redirect, request, url_for
from quokka.admin.views import ModelView
from quokka.admin.forms import Form, fields
from werkzeug.security import check_password_hash, generate_password_hash
from flask_simplelogin import SimpleLogin


def create_user(**data):
    if 'username' not in data or 'password' not in data:
        raise ValueError('username and password are required.')

    data['_id'] = data['username']
    data['password'] = generate_password_hash(
        data.pop('password'),
        method='pbkdf2:sha256'
    )
    current_app.db.users.insert_one(data)
    return data


class UserForm(Form):
    username = fields.StringField('Username')
    email = fields.StringField('Email')
    password = fields.PasswordField('Password')


class UserView(ModelView):
    column_list = ('username', 'email')
    column_sortable_list = ('username', 'email')

    form = UserForm

    page_size = 20
    can_set_page_size = True

    # Correct user_id reference before saving
    def on_model_change(self, form, model):
        model['_id'] = model.get('username')
        # todo reencrypt password if changed
        return model


def validate_login(user):
    db_user = current_app.db.users.find_one({"_id": user['username']})
    if not db_user:
        return False
    if check_password_hash(db_user['password'], user['password']):
        return True
    return False


def configure(app):
    SimpleLogin(app, login_checker=validate_login)


def configure_user_admin(app):
    app.admin.register(
        app.db.users,
        UserView,
        name='Users',
        category='Administration'
    )



# coding: utf-8

import os

from flask import current_app, request, send_from_directory
from flask_security import roles_accepted
from quokka.core.views import (ContentDetail, ContentDetailPreview,
                               ContentList, TagList)
from quokka.views import FeedAtom, FeedRss, SiteMap, TagAtom, TagRss


# from quokka.core.models.channel import Channel


@roles_accepted('admin', 'developer')
def template_files(filename):
    template_path = os.path.join(current_app.root_path,
                                 current_app.template_folder)
    return send_from_directory(template_path, filename)


@roles_accepted('admin', 'developer')
def theme_template_files(identifier, filename):
    template_path = os.path.join(
        current_app.root_path,
        "themes",
        identifier,
        "templates"
    )
    return send_from_directory(template_path, filename)


def media(filename):
    return send_from_directory(current_app.config.get('MEDIA_ROOT'), filename)


def static_from_root():
    return send_from_directory(current_app.static_folder, request.path[1:])


def configure(app):
    app.add_quokka_url_rule('/sitemap.xml',
                            view_func=SiteMap.as_view('sitemap'))
    app.add_quokka_url_rule('/mediafiles/<path:filename>', view_func=media)
    app.add_quokka_url_rule('/template_files/<path:filename>',
                            view_func=template_files)
    app.add_quokka_url_rule(
        '/theme_template_files/<identifier>/<path:filename>',
        view_func=theme_template_files
    )
    for filepath in app.config.get('MAP_STATIC_ROOT', []):
        app.add_quokka_url_rule(filepath, view_func=static_from_root)

    # Match content detail, .html added to distinguish from channels
    # better way? how?
    content_extension = app.config.get("CONTENT_EXTENSION", "html")
    app.add_quokka_url_rule('/<path:long_slug>.{0}'.format(content_extension),
                            view_func=ContentDetail.as_view('detail'))

    # Draft preview
    app.add_quokka_url_rule('/<path:long_slug>.preview',
                            view_func=ContentDetailPreview.as_view('preview'))

    # Atom Feed
    app.add_quokka_url_rule(
        '/<path:long_slug>.atom',
        view_func=FeedAtom.as_view('atom_list')
    )
    app.add_quokka_url_rule(
        '/tag/<tag>.atom', view_func=TagAtom.as_view('atom_tag')
    )

    # RSS Feed
    app.add_quokka_url_rule(
        '/<path:long_slug>.xml', view_func=FeedRss.as_view('rss_list')
    )
    app.add_quokka_url_rule('/tag/<tag>.xml',
                            view_func=TagRss.as_view('rss_tag'))

    # Tag list
    app.add_quokka_url_rule('/tag/<tag>/', view_func=TagList.as_view('tag'))

    # Match channels by its long_slug mpath
    app.add_quokka_url_rule('/<path:long_slug>/',
                            view_func=ContentList.as_view('list'))
    # Home page
    app.add_quokka_url_rule(
        '/',
        view_func=ContentList.as_view('home'),
        defaults={"long_slug": Channel.get_homepage('long_slug') or "home"}
    )



# coding: utf-8
# TODO: adapt to tinymongo

# from flask_admin.contrib.mongoengine.ajax import QueryAjaxModelLoader
from flask_admin.model.ajax import DEFAULT_PAGE_SIZE


class AjaxModelLoader(object):  #QueryAjaxModelLoader):
    """
    """
    def __init__(self, name, model, **options):
        self.filters = options.pop('filters', None)
        super(AjaxModelLoader, self).__init__(name, model, **options)

    def get_list(self, term, offset=0, limit=DEFAULT_PAGE_SIZE):
        query = self.model.objects

        criteria = None

        for field in self._cached_fields:
            flt = {u'%s__icontains' % field.name: term}

            if not criteria:
                criteria = mongoengine.Q(**flt)
            else:
                criteria |= mongoengine.Q(**flt)

        query = query.filter(criteria)

        if self.filters:
            query = query.filter(**self.filters)

        if offset:
            query = query.skip(offset)

        return query.limit(limit).all()



# coding: utf-8

from shiftpy.wsgi_utils import envify


def activate(app):
    # If running on Openshift wraps its virtualenv, otherwise do nothing
    envify(app)
    # Space left to insert hooks to another PaaS if needed
    return app



# coding: utf-8
import datetime as dt

from flask import current_app
from flask_admin.helpers import get_form_data
from quokka.admin.forms import READ_ONLY, Form, fields, rules, validators
from quokka.admin.utils import _
from werkzeug.utils import import_string


# Utils

def get_content_formats(instances=False):
    content_formats = current_app.config.get(
        'CONTENT_FORMATS',
        {
          'markdown': {
              'choice_text': 'Markdown',
              'help_text': 'Markdown text editor',
              'content_format_class': 'quokka.core.content_formats.MarkdownFormat'  # noqa
          }
        }
    )
    if instances:
        for identifier, data in content_formats:
            data['content_format_instance'] = import_string(
                data['content_format_class']
            )()
    return content_formats


def get_content_format_choices():
    content_formats = get_content_formats()
    return [
        # ('value', 'TEXT')
        (identifier, data['choice_text'])
        for identifier, data
        in content_formats.items()
    ]


def get_format(obj):
    content_formats = get_content_formats()
    try:
        obj_content_format = content_formats[obj['content_format']]
        content_format = import_string(
            obj_content_format['content_format_class']
        )()
        return content_format
    except (KeyError):
        return PlainFormat()


def get_edit_form(obj):
    return get_format(obj).get_edit_form(obj)


def validate_category(form, field):
    if field.data is not None:
        items = field.data.split(',')
        if len(items) > 1:
            return _(u'You can select only one category')

# classes


class BaseForm(Form):

    title = fields.StringField(_('Title'), [validators.required()])
    # todo: validade existing category/title
    summary = fields.TextAreaField(_('Summary'))
    category = fields.Select2TagsField(
        _('Category'),
        [validators.CallableValidator(validate_category)],
        save_as_list=False,
        render_kw={'data-tags': '["hello", "world"]'},
        # todo: ^ settings.default_categories + db_query
        default='general'
        # todo: default should come from settings
    )
    authors = fields.Select2TagsField(
        _('Authors'),
        [validators.required()],
        save_as_list=True,
        render_kw={'data-tags': '["Bruno Rocha", "Karla Magueta"]'},
        # todo: settings.default_authors + current + db_query
        default=['Bruno Rocha']
        # todo: default should be current user if auth else O.S user else ?
    )


class CreateForm(BaseForm):
    """Default create form where content format is chosen"""
    content_type = fields.SelectField(
        _('Type'),
        [validators.required()],
        choices=[('article', _('Article')), ('page', _('Page'))]
    )
    content_format = fields.SmartSelect2Field(
        _('Format'),
        [validators.required()],
        choices=get_content_format_choices
    )


class BaseEditForm(BaseForm):
    """Edit form with all missing fields except `content`"""

    content_type = fields.PassiveStringField(
        _('Type'),
        render_kw=READ_ONLY
    )
    content_format = fields.PassiveStringField(_('Format'), render_kw=READ_ONLY)

    tags = fields.Select2TagsField(_('Tags'), save_as_list=True)
    # todo: ^ provide settings.default_tags + db_query
    date = fields.DateTimeField(
        _('Date'),
        [validators.required()],
        default=dt.datetime.now
    )
    # todo: ^default should be now
    modified = fields.HiddenField(_('Modified'))
    # todo: ^populate on save
    slug = fields.StringField(_('Slug'))
    # todo: create based on category / title
    language = fields.SmartSelect2Field(
        _('Language'),
        choices=lambda: [
            (lng, lng)
            for lng in current_app.config.get('BABEL_LANGUAGES', ['en'])
        ]
    )
    translations = fields.HiddenField(_('Translations'))
    # todo: ^ create action 'add translation'
    published = fields.BooleanField(
        _('Status'),
        render_kw={
            'data-toggle': "toggle",
            'data-on': _("Published"),
            'data-off': _("Draft"),
            "data-onstyle": 'success'
        }
    )
    # todo: ^ published | draft


class BaseFormat(object):
    identifier = None
    edit_form = BaseEditForm
    form_rules = None

    def get_edit_form(self, obj):
        return self.edit_form(get_form_data(), **obj)

    def get_identifier(self):
        return self.identifier or self.__class__.__name__

    def get_form_rules(self):
        if self.form_rules is not None:
            self.form_rules.append(
                rules.Field(
                    'csrf_token',
                    render_field='quokka_macros.render_hidden_field'
                )
            )
        return self.form_rules

    def before_save(self, form, model, is_created):
        """optional"""

    def after_save(self, form, model, is_created):
        """optional"""

    def extra_js(self):
        return []


# Customs


class PlainEditForm(BaseEditForm):
    content = fields.TextAreaField(_('Plain Content'))


class PlainFormat(BaseFormat):
    edit_form = PlainEditForm


class HTMLEditForm(BaseEditForm):
    content = fields.TextAreaField(_('HTML Content'))


class HTMLFormat(BaseFormat):
    edit_form = HTMLEditForm


class MarkdownEditForm(BaseEditForm):
    content = fields.TextAreaField(_('Markdown Content'))


class MarkdownFormat(BaseFormat):
    edit_form = MarkdownEditForm
    form_rules = [
        rules.FieldSet(('title', 'summary')),
        rules.Field('content'),
        rules.FieldSet(('category', 'authors', 'tags')),
        rules.FieldSet(('date', 'language')),
        rules.FieldSet(('slug', 'content_type', 'content_format')),
        rules.Field('published')
    ]

    def before_save(self, form, model, is_created):
        print('before save')

    def after_save(self, form, model, is_created):
        print('after save')

    def extra_js(self):
        return []



# coding: utf-8
"""This core extension cannot be loaded from settings.yml
should be loaded directly in create_app"""

from dynaconf.contrib import FlaskDynaconf
from quokka.config import settings


def configure_dynaconf(app):
    FlaskDynaconf(app, dynaconf_instance=settings)



"""Quokka CMS!"""

__version__ = '0.3.0'

from quokka.app import QuokkaApp
from quokka.core import configure_extensions, configure_extension
from quokka.core.flask_dynaconf import configure_dynaconf
# from quokka.core.db import configure_db


def create_app_base(test=False, ext_list=None, **settings):
    """Creates basic app only with extensions provided in ext_list
    useful for testing."""

    app = QuokkaApp('quokka')
    configure_dynaconf(app)
    # configure_db(app)
    if settings:
        app.config.update(settings)

    if test or app.config.get('TESTING'):
        app.testing = True

    if ext_list:
        for ext in ext_list:
            configure_extension(ext, app=app)

    return app


def create_app(test=False, **settings):
    """Creates full app with all extensions loaded"""
    app = create_app_base(test=test, **settings)
    configure_extensions(app)
    return app



# coding: utf-8


def expose(url='/', methods=('GET',)):
    """
        Use this decorator to expose views in your view classes.

        `url`
            Relative URL for the view
        `methods`
            Allowed HTTP methods. By default only GET is allowed.
    """
    def wrap(f):
        if not hasattr(f, '_urls'):
            f._urls = []
        f._urls.append((url, methods))
        return f

    return wrap



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from quokka.utils.settings import get_setting_value

from pyshorteners.shorteners import Shortener


class ShorterURL(object):

    __shortener = None

    @property
    def shortener(self):
        if not self.__shortener:
            shortener_config = get_setting_value(
                'SHORTENER_SETTINGS', {}).copy()
            shortener_name = shortener_config.pop('name')
            self.__shortener = Shortener(shortener_name, **shortener_config)
        return self.__shortener

    def short(self, url):
        url = url.replace('localhost', '127.0.0.1')
        return self.shortener.short(url)



# coding: utf-8

# TODO load from blueprints

import sys

from flask import Blueprint
from pymongo.mongo_client import MongoClient
from quokka.core.app import QuokkaModule
from quokka.core.models.content import Content
from quokka_themes import Theme
from werkzeug.routing import Rule

if sys.version_info.major == 3:
    unicode = lambda x: u'{}'.format(x)  # noqa  # flake8: noqa
    basestring = str  # noqa  # flake8: noqa

basetypes = (
    int, str, float, dict, list, tuple,
    Blueprint, QuokkaModule, Theme, Rule, MongoClient,
    basestring, unicode
)


def is_instance(v, cls):
    cls_map = {
        tp.__name__: tp for tp in basetypes
    }
    return isinstance(v, cls_map.get(cls, str))


def get_content(**kwargs):
    try:
        return Content.objects.get(**kwargs)
    except:
        return None


def get_contents(limit=None, order_by="-created_at", **kwargs):
    contents = Content.objects.filter(**kwargs).order_by(order_by)
    if limit:
        contents = contents[:limit]
    return contents


def configure(app):
    app.jinja_env.filters['isinstance'] = is_instance
    app.add_template_global(get_content)
    app.add_template_global(get_contents)



# coding: utf-8
import random
import sys

from flask import current_app
from flask_admin import form
from flask_admin.form.upload import ImageUploadInput
from werkzeug.datastructures import FileStorage


class SmartSelect2Field(form.Select2Field):

    def iter_choices(self):
        """
        We should update how choices are iter to make sure that value from
        internal list or tuple should be selected.
        """
        if self.allow_blank:
            yield (u'__None', self.blank_text, self.data is None)

        for value, label in self.concrete_choices:
            yield (value, label, self.coerce(value) == self.data)

        # for value, label in self.concrete_choices:
        #     yield (value, label, (self.coerce, self.data))

    @property
    def concrete_choices(self):
        if callable(self.choices):
            return self.choices()
        return self.choices

    @property
    def choice_values(self):
        return [value for value, label in self.concrete_choices]

    def pre_validate(self, form):
        if self.allow_blank and self.data is None:
            return

        values = self.choice_values
        if (self.data is None and u'' in values) or self.data in values:
            return True

        super(SmartSelect2Field, self).pre_validate(form)


class ThumbWidget(ImageUploadInput):
    empty_template = ""
    data_template = ('<div class="image-thumbnail">'
                     ' <img %(image)s>'
                     '</div>')


class ThumbField(form.ImageUploadField):
    widget = ThumbWidget()


# class ImageUploadField(form.ImageUploadField):
#     def is_file_allowed(self, filename):
#         extensions = self.allowed_extensions  # noqa
#         if isinstance(extensions, (str, unicode)) and extensions.isupper():
#             items = current_app.config.get(extensions, extensions)
#             merged_items = [
#                 item.lower() for item in items
#             ] + [item.upper() for item in items]
#             self.allowed_extensions = merged_items
#         return super(ImageUploadField, self).is_file_allowed(filename)


class ContentImageField(form.ImageUploadField):

    def populate_obj(self, obj, name):
        pass

    # def save_contents(self, obj):
    #     # field = getattr(obj, name, None)
    #     # if field:
    #     #     # If field should be deleted, clean it up
    #     #     if self._should_delete:
    #     #         self._delete_file(field)
    #     #         setattr(obj, name, None)
    #     #         return

    #     new_image = Image(
    #         title=u"Image: {0}".format(obj.title),
    #         slug=u"{0}-{1}".format(obj.slug, random.getrandbits(8)),
    #         channel=Image.get_default_channel(),
    #         published=True
    #     )
    #     if self.data and isinstance(self.data, FileStorage):
    #         # if field:
    #         #     self._delete_file(field)

    #         filename = self.generate_name(new_image, self.data)
    #         filename = self._save_file(self.data, filename)

    #         setattr(new_image, 'path', filename)

    #         new_image.save()

    #         if obj.contents.filter(identifier='mainimage'):
    #             purpose = SubContentPurpose.objects.get(identifier='image')
    #         else:
    #             purpose = SubContentPurpose.objects.get(
    #                 identifier='mainimage'
    #             )

    #         subcontent = SubContent(
    #             content=new_image,
    #             purpose=purpose,
    #         )

    #         obj.contents.append(subcontent)
    #         obj.save()




import datetime
import json
import random

from flask import Response, flash, redirect, url_for
from flask_admin.actions import action
from quokka.admin.utils import _, _l, _n


class PublishAction(object):
    @action(
        'toggle_publish',
        _l('Publish/Unpublish'),
        _l('Publish/Unpublish?')
    )
    def action_toggle_publish(self, ids):
        for i in ids:
            instance = self.get_instance(i)
            instance.published = not instance.published
            instance.save()
        count = len(ids)
        flash(_n('Item successfully published/Unpublished.',
                 '%(count)s items were successfully published/Unpublished.',
                 count,
                 count=count))


class CloneAction(object):
    @action(
        'clone_item',
        _l('Create a copy'),
        _l('Are you sure you want a copy?')
    )
    def action_clone_item(self, ids):
        if len(ids) > 1:
            flash(
                _("You can select only one item for this action"),
                'error'
            )
            return

        instance = self.get_instance(ids[0])
        new = instance.from_json(instance.to_json())
        new.id = None
        new.published = False
        new.last_updated_by = None  # User.objects.get(id=current_user.id)
        new.updated_at = datetime.datetime.now()
        new.slug = "{0}-{1}".format(new.slug, random.getrandbits(32))
        new.save()
        return redirect(url_for('.edit_view', id=new.id))


class ExportAction(object):
    @action('export_to_json', _l('Export as json'))
    def export_to_json(self, ids):
        qs = self.model.objects(id__in=ids)

        return Response(
            qs.to_json(),
            mimetype="text/json",
            headers={
                "Content-Disposition":
                "attachment;filename=%s.json" % self.model.__name__.lower()
            }
        )

    @action('export_to_csv', _l('Export as csv'))
    def export_to_csv(self, ids):
        qs = json.loads(self.model.objects(id__in=ids).to_json())

        def generate():
            yield ','.join(list(max(qs, key=lambda x: len(x)).keys())) + '\n'
            for item in qs:
                yield ','.join([str(i) for i in list(item.values())]) + '\n'

        return Response(
            generate(),
            mimetype="text/csv",
            headers={
                "Content-Disposition":
                "attachment;filename=%s.csv" % self.model.__name__.lower()
            }
        )



from dynaconf import LazySettings

settings = LazySettings(
    ENVVAR_FOR_DYNACONF="QUOKKA_SETTINGS_MODULE",
    DYNACONF_NAMESPACE='QUOKKA',
    SETTINGS_MODULE_FOR_DYNACONF='settings.yml',
    YAML='.secrets.yml'  # extra yaml file override ^
)



# coding: utf -8

from flask_admin.babel import gettext, lazy_gettext, ngettext


def _(*args, **kwargs):
    return gettext(*args, **kwargs)


def _l(*args, **kwargs):
    return lazy_gettext(*args, **kwargs)


def _n(*args, **kwargs):
    return ngettext(*args, **kwargs)



from babel.support import LazyProxy
from flask import g
from flask_babelex import gettext, lazy_gettext, ngettext


# from quokka.utils.translations import ugettext_lazy as _


def ugettext(s):
    # we assume a before_request function
    # assigns the correct user-specific
    # translations
    return g.translations.ugettext(s)

ugettext_lazy = LazyProxy(ugettext)

_ = gettext
_l = lazy_gettext
_n = ngettext



# coding: utf-8

from flask_admin.babel import Translations
from flask_admin.form import rules  # noqa
from flask_admin.form.fields import (DateTimeField, JSONField, Select2Field,
                                     Select2TagsField, TimeField)
from flask_admin.form.widgets import Select2TagsWidget
from flask_admin.model.fields import InlineFieldList, InlineFormField
from flask_wtf import FlaskForm
from quokka.admin.fields import SmartSelect2Field
from quokka.admin.wtforms_html5 import AutoAttrMeta
from wtforms import fields as _fields
from wtforms import widgets as _widgets
from wtforms import validators  # noqa
from wtforms.validators import ValidationError


# from wtforms_components import read_only  # noqa
# from wtforms_components import ReadOnlyWidgetProxy  # noqa


class PassiveField(object):
    """
    Passive field that does not populate obj values.
    """
    def populate_obj(self, obj, name):
        pass


class PassiveHiddenField(PassiveField, _fields.HiddenField):
    pass


class PassiveStringField(PassiveField, _fields.StringField):
    pass


fields = _fields  # noqa
fields.SmartSelect2Field = SmartSelect2Field
fields.DateTimeField = DateTimeField
fields.TimeField = TimeField
fields.Select2Field = Select2Field
fields.Select2TagsField = Select2TagsField
fields.JSONField = JSONField
fields.InlineFieldList = InlineFieldList
fields.InlineFormField = InlineFormField
fields.PassiveHiddenField = PassiveHiddenField
fields.PassiveStringField = PassiveStringField

widgets = _widgets
widgets.Select2TagsWidget = Select2TagsWidget


READ_ONLY = {'readonly': True}


class Form(FlaskForm):
    """Base class to customize wtforms"""
    _translations = Translations()
    Meta = AutoAttrMeta

    def _get_translations(self):
        return self._translations


class CallableValidator(object):
    """
    Takes a callable and validates using it
    """
    def __init__(self, function, message=None):
        self.function = function
        self.message = message

    def __call__(self, form, field):
        validation = self.function(form, field)
        if validation is not None:
            raise ValidationError(self.message or validation)


validators.CallableValidator = CallableValidator



# coding: utf -8

from flask import current_app, redirect
# from flask_admin.babel import gettext, ngettext
from flask_admin import AdminIndexView
from flask_admin.contrib.fileadmin import FileAdmin as _FileAdmin
from flask_admin.contrib.pymongo import ModelView as PyMongoModelView
# from flask_admin import helpers as h
from flask_login import current_user, login_url
from quokka.admin.actions import CloneAction, PublishAction
# from quokka.template import render_template
# from quokka.admin.widgets import PrepopulatedText
# from quokka.admin.fields import ContentImageField
# from quokka.utils.upload import dated_path, lazy_media_path
from quokka.utils.routing import expose


# from flask_admin import BaseView as AdminBaseView

# from quokka.admin.fields import ThumbField

# from quokka.admin import formatters


class ThemeMixin(object):
    pass
    # def render(self, template, **kwargs):
    #     # Store self as admin_view
    #     kwargs['admin_view'] = self
    #     kwargs['admin_base_template'] = self.admin.base_template
    #     # Provide i18n support even if flask-babel is not installed or enabled.
    #     kwargs['_gettext'] = gettext
    #     kwargs['_ngettext'] = ngettext
    #     kwargs['h'] = h
    #     # Contribute extra arguments
    #     kwargs.update(self._template_args)
    #     theme = current_app.config.get('ADMIN_THEME')
    #     return render_template(template, theme=theme, **kwargs)

    # def render(self, template, **kwargs):
    #     """
    #         Render template
    #         :param template:
    #             Template path to render
    #         :param kwargs:
    #             Template arguments
    #     """
    #     # Store self as admin_view
    #     kwargs['admin_view'] = self
    #     kwargs['admin_base_template'] = self.admin.base_template

    #     # Provide i18n support even if flask-babel is not installed
    #     # or enabled.
    #     kwargs['_gettext'] = babel.gettext
    #     kwargs['_ngettext'] = babel.ngettext
    #     kwargs['h'] = h

    #     # Expose get_url helper
    #     kwargs['get_url'] = self.get_url

    #     # Expose config info
    #     kwargs['config'] = current_app.config

    #     # Contribute extra arguments
    #     kwargs.update(self._template_args)

    #     return render_template(template, **kwargs)


class RequiresLogin(object):

    def _handle_view(self, *args, **kwargs):  # noqa
        """Admin views requires login"""
        if current_app.config.get('ADMIN_REQUIRES_LOGIN') is True:
            if not current_user.is_authenticated:
                return redirect(login_url('quokka.login', next_url="/admin"))


class FileAdmin(ThemeMixin, RequiresLogin, _FileAdmin):

    def __init__(self, *args, **kwargs):
        self.editable_extensions = kwargs.pop('editable_extensions', tuple())
        super(FileAdmin, self).__init__(*args, **kwargs)


class IndexView(RequiresLogin, ThemeMixin, AdminIndexView):

    @expose('/')
    def index(self):
        return self.render('admin/index.html')


class ModelView(CloneAction, PublishAction,
                ThemeMixin, RequiresLogin,
                PyMongoModelView):
    """Base model view for all contents"""

    page_size = 20
    can_set_page_size = True

    # form_subdocuments = {}
    # datetime_format = "%Y-%m-%d %H:%M"
    # formatters = {
    #     'datetime': formatters.format_datetime,
    #     'view_on_site': formatters.view_on_site,
    #     'ul': formatters.format_ul,
    #     'status': formatters.format_status,
    #     'get_url': formatters.get_url,
    #     'link': formatters.format_link
    # }
    # column_formatters_args = {}

    # def get_datetime_format(self):
    #     return current_app.config.get('DATETIME_FORMAT', self.datetime_format)


# class BaseContentView(ModelView):
#     """
#     All attributes added here for example
#     more info in admin source
#     github.com/mrjoes/flask-admin/blob/master/flask_admin/model/base.py
#     or Flask-admin documentation
#     """

#     roles_accepted = ('admin', 'editor', 'author')
#     can_create = True
#     can_edit = True
#     can_delete = True

#     # list_template = 'admin/model/list.html'
#     # edit_template = 'admin/custom/edit.html'
#     # create_template = 'admin/custom/create.html'

#     column_list = (
#         'title', 'slug', 'channel', 'published', 'created_at',
#         'available_at', 'view_on_site'
#     )

#     column_formatters = {
#         'view_on_site': formatters.view_on_site,
#         'created_at': formatters.format_datetime,
#         'available_at': formatters.format_datetime
#     }

#     # column_type_formatters = {}
#     # column_labels = {}
#     # column_descriptions = {}
#     # column_sortable_list = [] / ('name', ('user', 'user.username'))
#     # column_default_sort = 'pk'
#     # column_choices = {'column': ('value', 'display')}
#     # column_display_pk = True

#     column_filters = ['published', 'title', 'summary',
#                       'created_at', 'available_at']
#     column_searchable_list = ('title', 'summary')

#     form_columns = ['title', 'slug', 'channel', 'related_channels', 'summary',
#                     'published', 'add_image', 'contents',
#                     'show_on_channel', 'available_at', 'available_until',
#                     'tags', 'values', 'template_type', 'license', 'authors']
#     # form_excluded_columns = []
#     # form = None
#     # form_overrides = None

#     form_widget_args = {
#         'summary': {
#             'style': 'width: 400px; height: 100px;'
#         },
#         'title': {'style': 'width: 400px'},
#         'slug': {'style': 'width: 400px'},
#     }

#     form_args = {
#         # 'body': {'widget': TextEditor()},
#         'slug': {'widget': PrepopulatedText(master='title')}
#     }

#     form_subdocuments = {
#         'contents': {
#             'form_subdocuments': {
#                 None: {
#                     'form_columns': ('content', 'caption', 'purpose',
#                                      'order', 'thumb'),
#                     'form_ajax_refs': {
#                         'content': {
#                             'fields': ['title', 'long_slug', 'summary']
#                         }
#                     },
#                     'form_extra_fields': {
#                         'thumb': ThumbField('thumb', endpoint="media")
#                     }
#                 }
#             }
#         },
#     }
#     # form_extra_fields = {}
#     form_extra_fields = {
#         'add_image': ContentImageField(
#             'Add new image',
#             base_path=lazy_media_path(),
#             # thumbnail_size=get_setting_value('MEDIA_IMAGE_THUMB_SIZE',
#             #                                  default=(100, 100, True)),
#             endpoint="media",
#             namegen=dated_path,
#             permission=0o777,
#             allowed_extensions="MEDIA_IMAGE_ALLOWED_EXTENSIONS",
#         )
#     }

#     # action_disallowed_list
#     # page_size = 20
#     # form_ajax_refs = {
#     #     'main_image': {"fields": ('title',)}
#     # }

#     # def get_list_columns(self):
#     #     column_list = super(BaseContentAdmin, self).get_list_columns()
#     #     if get_setting_value('SHORTENER_ENABLED'):
#     #         column_list += [('short_url', 'Short URL')]
#     #     return column_list

#     def after_model_change(self, form, model, is_created):
#         if not hasattr(form, 'add_image'):
#             return

#         form.add_image.save_contents(model)



#!/usr/bin/env python
# coding: utf-8

"""
This functions created by Alvaro Justen (Turicas)
"""
from slugify import slugify as awesome_slugify


def slugify(text):
    return awesome_slugify(text).lower()


def abbreviate(name, pretty=False):
    names = name.split()
    if len(names) == 1:
        return name
    if len(names) == 2:
        return name
    result = [names[0]]
    tiny_name = False
    for surname in names[1:-1]:
        if len(surname) <= 3:
            result.append(surname)
            tiny_name = True
        else:
            if pretty and tiny_name:
                result.append(surname)
            else:
                result.append(surname[0] + '.')
            tiny_name = False
    result.append(names[-1])
    return ' '.join(result)



# coding: utf-8

# from flask_admin.helpers import get_form_data
import datetime as dt
from flask import current_app
from quokka.admin.forms import ValidationError
from quokka.admin.utils import _
from quokka.admin.views import ModelView
from quokka.core.content_formats import CreateForm, get_format
from quokka.utils.text import slugify


class ContentView(ModelView):
    """Base form for all contents"""
    # TODO: move to base class and read from settings
    details_modal = True
    can_view_details = True
    create_modal = True
    can_export = True
    export_types = ['csv', 'json', 'yaml', 'html', 'xls']

    details_modal_template = 'admin/model/modals/details.html'
    # create_template = 'admin/model/create.html'

    edit_template = 'admin/quokka/edit.html'
    # TODO: ^get edit_template from content_type

    page_size = 20
    can_set_page_size = True

    form = CreateForm
    column_list = (
        'title',
        'category',
        'authors',
        'date',
        'modified',
        'lang',
        'status'
    )

    column_sortable_list = (
        'title',
        'category',
        'authors',
        'date',
        'modified',
        'lang',
        'status'
    )
    # column_default_sort = 'date'

    # TODO: implement scaffold_list_form in base class
    # column_editable_list = ['category', 'status', 'title']

    column_details_list = ['content_format']
    # column_export_list = []
    # column_formatters_export
    # column_formatters = {fieldname: callable} - view, context, model, name

    column_extra_row_actions = None
    """
        List of row actions (instances of :class:`~flask_admin.model.template.BaseListRowAction`).

        Flask-Admin will generate standard per-row actions (edit, delete, etc)
        and will append custom actions from this list right after them.

        For example::

            from flask_admin.model.template import EndpointLinkRowAction, LinkRowAction

            class MyModelView(BaseModelView):
                column_extra_row_actions = [
                    LinkRowAction('glyphicon glyphicon-off', 'http://direct.link/?id={row_id}'),
                    EndpointLinkRowAction('glyphicon glyphicon-test', 'my_view.index_view')
                ]
    """

    # form_edit_rules / form_create_rules
    # form_rules = [
    #     # Define field set with header text and four fields
    #     rules.FieldSet(('title', 'category', 'tags'), 'Base'),
    #     # ... and it is just shortcut for:
    #     rules.Header('Content Type'),
    #     rules.Field('summary'),
    #     rules.Field('date'),
    #     # ...
    #     # It is possible to create custom rule blocks:
    #     # MyBlock('Hello World'),
    #     # It is possible to call macros from current context
    #     # rules.Macro('my_macro', foobar='baz')
    # ]

    # def create_form(self):
    #     form = super(ContentView, self).create_form()
    #     form.content_type.choices = [('a', 'a'), ('b', 'b')]
    #     return form

    # @property
    # def extra_js(self):
    #     return [
    #         url_for('static', filename='js/quokka_admin.js')
    #     ]

    def edit_form(self, obj):
        content_format = get_format(obj)
        self.form_edit_rules = content_format.get_form_rules()
        self._refresh_form_rules_cache()
        return content_format.get_edit_form(obj)

    def get_save_return_url(self, model, is_created):
        if is_created:
            return self.get_url('.edit_view', id=model['_id'])
        return super(ContentView, self).get_save_return_url(model, is_created)

    def on_model_change(self, form, model, is_created):
        # check if exists

        existent = current_app.db.index.find_one(
            {'title': model['title'], 'category': model['category']}
        )

        duplicate_error_message = u'{0} "{1}/{2}" {3}'.format(
            _('duplicate error:'),
            model['category'],
            model['title'],
            _('already exists.')
        )

        if (is_created and existent) or (
                existent and existent['_id'] != model['_id']):
            raise ValidationError(duplicate_error_message)

        if is_created:
            model['date'] = dt.datetime.now()
            model['slug'] = slugify(model['title'])
        else:
            model['modified'] = dt.datetime.now()

        get_format(model).before_save(form, model, is_created)

    def after_model_change(self, form, model, is_created):
        # TODO: Spawn async process for this.
        # update tags, categories and authors
        get_format(model).after_save(form, model, is_created)


def configure(app):
    app.admin.register(
        app.db.index,
        ContentView,
        name=_('Content'),
        endpoint='contentview'
    )
    return 'content'



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['quokka', 'quokka.admin', 'quokka.core', 'quokka.utils']

package_data = \
{'': ['*'],
 'quokka': ['translations/*',
            'translations/zh_CN/*',
            'translations/zh_CN/LC_MESSAGES/*',
            'babel/*',
            'templates/*',
            'templates/admin/*',
            'templates/admin/quokka/*',
            'templates/errors/*',
            'project_template/*',
            'project_template/databases/*',
            'project_template/uploads/*',
            'project_template/themes/*',
            'project_template/themes/pure/*',
            'project_template/themes/pure/templates/*',
            'project_template/themes/Flex/*',
            'project_template/themes/Flex/static/*',
            'project_template/themes/Flex/static/stylesheet/*',
            'project_template/themes/Flex/static/img/*',
            'project_template/themes/Flex/static/pygments/*',
            'project_template/themes/Flex/static/font-awesome/*',
            'project_template/themes/Flex/static/font-awesome/fonts/*',
            'project_template/themes/Flex/static/font-awesome/css/*',
            'project_template/themes/Flex/translations/*',
            'project_template/themes/Flex/translations/nl_NL/*',
            'project_template/themes/Flex/translations/nl_NL/LC_MESSAGES/*',
            'project_template/themes/Flex/translations/ru/*',
            'project_template/themes/Flex/translations/ru/LC_MESSAGES/*',
            'project_template/themes/Flex/translations/zh_CN/*',
            'project_template/themes/Flex/translations/zh_CN/LC_MESSAGES/*',
            'project_template/themes/Flex/translations/pt_BR/*',
            'project_template/themes/Flex/translations/pt_BR/LC_MESSAGES/*',
            'project_template/themes/Flex/translations/fr/*',
            'project_template/themes/Flex/translations/fr/LC_MESSAGES/*',
            'project_template/themes/Flex/translations/en/*',
            'project_template/themes/Flex/translations/en/LC_MESSAGES/*',
            'project_template/themes/Flex/translations/es/*',
            'project_template/themes/Flex/translations/es/LC_MESSAGES/*',
            'project_template/themes/Flex/translations/tr_TR/*',
            'project_template/themes/Flex/translations/tr_TR/LC_MESSAGES/*',
            'project_template/themes/Flex/translations/de/*',
            'project_template/themes/Flex/translations/de/LC_MESSAGES/*',
            'project_template/themes/Flex/translations/fa_IR/*',
            'project_template/themes/Flex/translations/fa_IR/LC_MESSAGES/*',
            'project_template/themes/Flex/translations/et/*',
            'project_template/themes/Flex/translations/et/LC_MESSAGES/*',
            'project_template/themes/Flex/translations/pl_PL/*',
            'project_template/themes/Flex/translations/pl_PL/LC_MESSAGES/*',
            'project_template/themes/Flex/translations/it/*',
            'project_template/themes/Flex/translations/it/LC_MESSAGES/*',
            'project_template/themes/Flex/translations/es_ES/*',
            'project_template/themes/Flex/translations/es_ES/LC_MESSAGES/*',
            'project_template/themes/Flex/translations/hu_HU/*',
            'project_template/themes/Flex/translations/hu_HU/LC_MESSAGES/*',
            'project_template/themes/Flex/templates/*',
            'project_template/themes/Flex/templates/partial/*',
            'project_template/themes/Flex/tests/*',
            'project_template/themes/Flex/tests/content/*',
            'project_template/themes/Flex/tests/content/stuff/*',
            'project_template/themes/Flex/tests/content/dev/*',
            'project_template/themes/Flex/tests/content/pages/*',
            'project_template/themes/Flex/pygments/*',
            'project_template/static_build/*']}

install_requires = \
['flask',
 'import_string',
 'flask_admin',
 'wtforms_components',
 'pymongo==2.4.1',
 'tinymongo',
 'tinydb_serialization',
 'manage',
 'dynaconf',
 'quokka_themes',
 'quokka_flask_htmlbuilder',
 'awesome_slugify',
 'flask_babelex',
 'flask_gravatar',
 'flask_mistune',
 'flask_simplelogin',
 'frozen_flask',
 'pillow',
 'pyrss2gen',
 'requests',
 'tablib',
 'flask_security']

entry_points = \
{'console_scripts': ['quokka = quokka.cli:main']}

setup(name='quokka',
      version='0.3.0',
      description='Quokka CMS!',
      author='Bruno Rocha',
      author_email='rochacbruno@gmail.com',
      url='https://github.com/rochacbruno/quokka',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      entry_points=entry_points,
      python_requires='>=3.6',
     )



# coding: utf-8
from inspect import getargspec
import import_string


def configure_extension(name, **kwargs):
    configurator = import_string(name)
    args = getargspec(configurator).args
    configurator(**{key: val for key, val in kwargs.items() if key in args})


def configure_extensions(app, admin=None):
    """Configure extensions provided in config file"""
    extensions = app.config.get(
        'CORE_EXTENSIONS', []
    ) + app.config.get(
        'EXTRA_EXTENSIONS', []
    )
    for configurator_name in extensions:
        configure_extension(configurator_name, app=app, admin=admin)
    return app



# coding: utf-8
import errno
import logging
import shutil
from pprint import pprint

import click
import yaml
from manage.cli import cli, init_cli
from manage.template import default_manage_dict
from quokka import create_app
# from quokka.core.security import User
from quokka.errors import DuplicateKeyError

app = create_app()


if app.config.get("LOGGER_ENABLED"):
    logging.basicConfig(
        level=getattr(logging, app.config.get("LOGGER_LEVEL", "DEBUG")),
        format=app.config.get(
            "LOGGER_FORMAT",
            '%(asctime)s %(name)-12s %(levelname)-8s %(message)s'),
        datefmt=app.config.get("LOGGER_DATE_FORMAT", '%d.%m %H:%M:%S')
    )


@cli.command()
@click.option('--reloader/--no-reloader', default=True)
@click.option('--debug/--no-debug', default=True)
@click.option('--host', default='127.0.0.1')
@click.option('--port', default=5000)
def runserver(reloader, debug, host, port):
    """Run the Flask development server i.e. app.run()"""
    app.run(
        use_reloader=reloader,
        debug=debug,
        host=host,
        port=port,
        extra_files=['settings.yml', '.secrets.yml']  # for reloader
    )


@cli.command()
def check():
    """Prints app status"""
    click.echo("Extensions.")
    pprint(app.extensions)
    click.echo("Modules.")
    pprint(app.blueprints)
    click.echo("App.")
    return app


@cli.command()
def showconfig():
    """click.echo all Quokka config variables"""
    from pprint import pprint
    click.echo("Config.")
    pprint(dict(app.config))


def copyfolder(src, dst):
    try:
        shutil.copytree(src, dst)
    except OSError as exc:
        if exc.errno == errno.ENOTDIR:
            shutil.copy(src, dst)
        else:
            raise


@cli.command()
@click.argument('name', required=True)
def init(name):
    """Initialize a new project in current folder\n
    $ quokka init mywebsite
    """
    folder_name = name.replace(' ', '_').lower()
    destiny = './{0}'.format(folder_name)
    copyfolder(
        '/home/brocha/Projects/quokka_ng/quokka/project_template',
        destiny
    )
    with open('{0}/manage.yml'.format(destiny), 'w') as manage_file:
        data = default_manage_dict
        data['project_name'] = name.title()
        manage_file.write(yaml.dump(data, default_flow_style=False))


@cli.command()
@click.option('--username', required=True)
@click.option('--email', required=True)
@click.option('--password', required=True)
def adduser(username, email, password):
    """Add new user with admin access"""
    # TODO: improve click.options to password
    try:
        User.create(username, email, password)
    except DuplicateKeyError as e:
        click.echo(str(e).replace('_id', 'username'))
    else:
        # TODO: get app_url dynamically
        app_url = 'http://localhost:5000/admin'
        click.echo('User {0} created!!! go to: {1}'.format(username, app_url))


def main():
    """
    Quokka CMS command line manager
    overwrites the manage loader
    """
    manager = init_cli(cli)
    # TODO: implement locked: to avoid manage to run
    return manager()  # from quokka.utils.populate import Populate



#!/usr/bin/python

import argparse
from werkzeug.serving import run_simple
from werkzeug.wsgi import DispatcherMiddleware
from quokka import create_app, create_api
from quokka.utils.paas import activate

application = DispatcherMiddleware(create_app(), {
    '/api': create_api()
})

application = app = activate(application)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Run Quokka App for WSGI")
    parser.add_argument('-p', '--port', help='App Port')
    parser.add_argument('-i', '--host', help='App Host')
    parser.add_argument('-r', '--reloader', action='store_true',
                        help='Turn reloader on')
    parser.add_argument('-d', '--debug', action='store_true',
                        help='Turn debug on')
    args = parser.parse_args()
    run_simple(
        args.host or '0.0.0.0',
        int(args.port) if args.port else 5000,
        application,
        use_reloader=args.reloader or False,
        use_debugger=args.debug or False,
    )



# coding: utf-8
import importlib
import os

from quokka.core.commands_collector import CommandsCollector


# def load_from_packages(app):
#     pass


def load_from_folder(app):
    """
        This code looks for any modules or packages in the given
        directory, loads them
        and then registers a blueprint
        - blueprints must be created with the name 'module'
        Implemented directory scan

        Bulk of the code taken from:
            https://github.com/smartboyathome/
               Cheshire-Engine/blob/master/ScoringServer/utils.py
    """
    blueprints_path = app.config.get('BLUEPRINTS_PATH', 'modules')
    path = os.path.join(
        app.config.get('PROJECT_ROOT', '..'),
        blueprints_path
    )
    base_module_name = ".".join([app.name, blueprints_path])
    dir_list = os.listdir(path)
    mods = {}
    object_name = app.config.get('BLUEPRINTS_OBJECT_NAME', 'module')
    module_file = app.config.get('BLUEPRINTS_MODULE_NAME', 'main')
    blueprint_module = module_file + '.py'
    for fname in dir_list:
        if not os.path.exists(os.path.join(path, fname, 'DISABLED')) and  \
                os.path.isdir(os.path.join(path, fname)) and \
                os.path.exists(os.path.join(path, fname, blueprint_module)):

            # register blueprint object
            module_root = ".".join([base_module_name, fname])
            module_name = ".".join([module_root, module_file])
            mods[fname] = importlib.import_module(module_name)
            blueprint = getattr(mods[fname], object_name)
            app.logger.info("registering blueprint: %s" % blueprint.name)
            app.register_blueprint(blueprint)

            # register admin
            try:
                importlib.import_module(".".join([module_root, 'admin']))
            except ImportError as e:
                app.logger.info(
                    "%s module does not define admin or error: %s", fname, e
                )

    app.logger.info("%s modules loaded", mods.keys())


def get_blueprint_commands(path, root, app_name):
    modules_path = os.path.join(root, path)
    base_module_name = ".".join([app_name, path])
    cmds = CommandsCollector(modules_path, base_module_name)
    return cmds


def blueprint_commands(app=None):
    return get_blueprint_commands(
        path=app.config.get('BLUEPRINTS_PATH', 'modules'),
        root=app.config.get('PROJECT_ROOT', '..'),
        app_name=app.name
    )



# coding: utf-8
import os.path as op
import random
from datetime import date

from flask import current_app
from speaklater import make_lazy_string
from werkzeug import secure_filename


def dated_path(obj, file_data):
    try:
        prefix = getattr(obj, 'model_name')
    except:
        prefix = "undefined"

    parts = op.splitext(file_data.filename)
    rand = random.getrandbits(16)
    filename = u"{name}_{rand}{ext}".format(
        rand=rand, name=parts[0], ext=parts[1]
    )
    filename = secure_filename(filename)
    today = date.today()
    path = u"{prefix}/{t.year}/{t.month}/{filename}".format(
        prefix=prefix, t=today, filename=filename
    )
    return path


def media_path(suffix=None):
    media_root = current_app.config.get('MEDIA_ROOT')
    if suffix:
        return op.join(media_root, suffix)
    return op.join(media_root)


def lazy_media_path(suffix=None):
    return make_lazy_string(lambda: media_path(suffix))



# coding: utf-8
from flask import Blueprint, Flask
from flask.helpers import _endpoint_from_view_func


class QuokkaApp(Flask):
    """
    Implements customizations on Flask
    - custom add_quokka_url_rule
    - properties to access db and admin
    """

    def add_quokka_url_rule(self, rule, endpoint=None,
                            view_func=None, **options):
        """Builds urls using quokka. prefix to avoid conflicts
        with external modules urls."""
        if endpoint is None:
            endpoint = _endpoint_from_view_func(view_func)
        if not endpoint.startswith('quokka.'):
            endpoint = 'quokka.' + endpoint
        self.add_url_rule(rule, endpoint, view_func, **options)

    @property
    def db(self):
        return self.extensions['db']

    @property
    def admin(self):
        ext = self.extensions['admin']
        return ext[0] if isinstance(ext, list) else ext


class QuokkaModule(Blueprint):
    """Overwrite blueprint namespace to quokka.modules.name
    to avoid conflicts with external Blueprints use same name"""

    def __init__(self, name, *args, **kwargs):
        name = "quokka.modules." + name
        super(QuokkaModule, self).__init__(name, *args, **kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*
import import_string
from flask_admin import Admin
from quokka.config import settings
from quokka.utils.translation import _l
from tinymongo import TinyMongoCollection

from .views import FileAdmin, IndexView, ModelView


class QuokkaAdmin(Admin):
    """Customizable admin"""
    registered = []

    def register(self, model, view=None, identifier=None, *args, **kwargs):
        """Register views in a simpler way
            admin.register(
                Link,
                LinkAdmin,
                category=_l("Content"),
                name=_l("Link")
            )
        """
        _view = view or ModelView

        if not identifier:
            if isinstance(model, TinyMongoCollection):
                identifier = '.'.join(
                    (view.__module__, view.__name__, model.tablename)
                )
            else:
                identifier = '.'.join((model.__module__, model.__name__))

            if identifier not in self.registered:
                self.add_view(_view(model, *args, **kwargs))
                self.registered.append(identifier)


def create_admin(app=None):
    """Admin factory"""
    index_view = IndexView()
    return QuokkaAdmin(
        app,
        index_view=index_view,
        template_mode=settings.get('FLASK_ADMIN_TEMPLATE_MODE')
    )


def configure_admin(app, admin=None):  # noqa
    """Configure admin extensions"""
    admin = admin or create_admin(app)

    custom_index = app.config.get('ADMIN_INDEX_VIEW')
    if custom_index:
        admin.index_view = import_string(custom_index)()
        if isinstance(admin._views[0], IndexView):  # noqa
            del admin._views[0]  # noqa
        admin._views.insert(0, admin.index_view)  # noqa

    admin_config = app.config.get(
        'ADMIN',
        {
            'name': 'Quokka Admin',
            'url': '/admin'
        }
    )

    for key, value in list(admin_config.items()):
        setattr(admin, key, value)

    # avoid registering twice
    if admin.app is None:
        admin.init_app(app)

    return admin


def configure_file_admin(app):
    for entry in app.config.get('FILE_ADMIN', []):
        try:
            app.admin.add_view(
                FileAdmin(
                    entry['path'],
                    entry['url'],
                    name=_l(entry['name']),
                    category=_l(entry['category']),
                    endpoint=entry['endpoint'],
                    editable_extensions=entry.get('editable_extensions')
                )
            )
        except Exception as e:
            app.logger.info(e)


def configure_extra_views(app):
    # adding extra views
    extra_views = app.config.get('ADMIN_EXTRA_VIEWS', [])
    for view in extra_views:
        app.admin.add_view(
            import_string(view['module'])(
                category=_l(view.get('category')),
                name=_l(view.get('name'))
            )
        )
